import { RequestHandler } from "express";
import { getJSONfromString } from "@/src/helpers/stringHelpers";
import { getAccountIdForRequest } from "@/src/services/loginService";
import { IMissionInventoryUpdateRequest } from "@/src/types/requestTypes";
import { addMissionInventoryUpdates, addMissionRewards } from "@/src/services/missionInventoryUpdateService";
import { getInventory } from "@/src/services/inventoryService";
import { getInventoryResponse } from "./inventoryController";
import { logger } from "@/src/utils/logger";
import { IMissionInventoryUpdateResponse } from "@/src/types/missionTypes";

/*
**** INPUT ****
- [ ]  crossPlaySetting
- [ ]  rewardsMultiplier
- [ ]  ActiveBoosters
- [x]  LongGuns
- [x]  Pistols
- [x]  Suits
- [x]  Melee
- [x]  RawUpgrades
- [x]  MiscItems
- [x]  RegularCredits
- [ ]  RandomUpgradesIdentified
- [ ]  MissionFailed
- [ ]  MissionStatus
- [ ]  CurrentLoadOutIds
- [ ]  AliveTime
- [ ]  MissionTime
- [x]  Missions
- [ ]  CompletedAlerts
- [ ]  LastRegionPlayed
- [ ]  GameModeId
- [ ]  hosts
- [x]  ChallengeProgress
- [ ]  SeasonChallengeHistory
- [ ]  PS (anticheat data)
- [ ]  ActiveDojoColorResearch
- [x]  RewardInfo
- [ ]  ReceivedCeremonyMsg
- [ ]  LastCeremonyResetDate
- [ ]  MissionPTS (Used to validate the mission/alive time above.)
- [ ]  RepHash
- [ ]  EndOfMatchUpload
- [ ]  ObjectiveReached
- [ ]  FpsAvg
- [ ]  FpsMin
- [ ]  FpsMax
- [ ]  FpsSamples
*/
//move credit calc in here, return MissionRewards: [] if no reward info
export const missionInventoryUpdateController: RequestHandler = async (req, res): Promise<void> => {
    const accountId = await getAccountIdForRequest(req);
    const missionReport = getJSONfromString<IMissionInventoryUpdateRequest>((req.body as string).toString());
    logger.debug("mission report:", missionReport);

    const inventory = await getInventory(accountId);
    const firstCompletion = missionReport.SortieId
        ? inventory.CompletedSorties.indexOf(missionReport.SortieId) == -1
        : false;
    const inventoryUpdates = await addMissionInventoryUpdates(inventory, missionReport);

    if (
        missionReport.MissionStatus !== "GS_SUCCESS" &&
        !(missionReport.RewardInfo?.jobId || missionReport.RewardInfo?.challengeMissionId)
    ) {
        await inventory.save();
        const inventoryResponse = await getInventoryResponse(inventory, true);
        res.json({
            InventoryJson: JSON.stringify(inventoryResponse),
            MissionRewards: []
        });
        return;
    }

    const { MissionRewards, inventoryChanges, credits, AffiliationMods, SyndicateXPItemReward } =
        await addMissionRewards(inventory, missionReport, firstCompletion);



    //TODO: figure out when to send inventory. it is needed for many cases.
    const response: IMissionInventoryUpdateResponse = {
        InventoryChanges: inventoryChanges,
        MissionRewards,
        ...credits,
        ...inventoryUpdates,
        //FusionPoints: inventoryChanges?.FusionPoints, // This in combination with InventoryJson or InventoryChanges seems to just double the number of endo shown, so unsure when this is needed.
        SyndicateXPItemReward,
        AffiliationMods
    };

    if (missionReport.ConquestMissionsCompleted !== undefined) {
        response.ConquestCompletedMissionsCount =
            missionReport.ConquestMissionsCompleted == 2 ? 0 : missionReport.ConquestMissionsCompleted + 1;



        let score = 1;
        if(missionReport.RewardInfo?.ConquestHardModeActive === 1)
            score += 3

        if (missionReport.RewardInfo?.ConquestPersonalModifiersActive !== undefined)
            score += missionReport.RewardInfo?.ConquestPersonalModifiersActive;
        if (missionReport.RewardInfo?.ConquestEquipmentSuggestionsFulfilled !== undefined)
            score += missionReport.RewardInfo?.ConquestEquipmentSuggestionsFulfilled;

        score *= missionReport.ConquestMissionsCompleted + 1

        if(missionReport.ConquestMissionsCompleted == 2 && missionReport.RewardInfo?.ConquestHardModeActive === 1)
            score += 1;

        const conquestType = missionReport.RewardInfo?.ConquestType;
        const conquestNode = conquestType == "HexConquest" ? "EchoesHexConquestHardModeUnlocked" : "EntratiLabConquestHardModeUnlocked";
        if(score >= 25 && inventory.NodeIntrosCompleted.find(x => x == conquestNode) === undefined)
            inventory.NodeIntrosCompleted.push(conquestNode);
        
        if(conquestType == "HexConquest")
        {
            inventory.EchoesHexConquestCacheScoreMission ??= 0;

            if(score > inventory.EchoesHexConquestCacheScoreMission )
                inventory.EchoesHexConquestCacheScoreMission = score;
        }else
        {
            inventory.EntratiLabConquestCacheScoreMission ??= 0;
    
            if(score > inventory.EntratiLabConquestCacheScoreMission )
                inventory.EntratiLabConquestCacheScoreMission = score;
        }
    }

    await inventory.save();
    const inventoryResponse = await getInventoryResponse(inventory, true);
    response.InventoryJson =  JSON.stringify(inventoryResponse);
    res.json(response);
};

/*
**** OUTPUT ****
- [x]  InventoryJson
- [x]  MissionRewards
- [x]  TotalCredits
- [x]  CreditsBonus
- [x]  MissionCredits
- [x]  InventoryChanges
- [x]  FusionPoints
*/
